(*********************************************************************************)
(*                Stog                                                           *)
(*                                                                               *)
(*    Copyright (C) 2012-2014 Maxence Guesdon. All rights reserved.              *)
(*                                                                               *)
(*    This program is free software; you can redistribute it and/or modify       *)
(*    it under the terms of the GNU General Public License as                    *)
(*    published by the Free Software Foundation, version 3 of the License.       *)
(*                                                                               *)
(*    This program is distributed in the hope that it will be useful,            *)
(*    but WITHOUT ANY WARRANTY; without even the implied warranty of             *)
(*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               *)
(*    GNU Library General Public License for more details.                       *)
(*                                                                               *)
(*    You should have received a copy of the GNU General Public                  *)
(*    License along with this program; if not, write to the Free Software        *)
(*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA                   *)
(*    02111-1307  USA                                                            *)
(*                                                                               *)
(*    As a special exception, you have permission to link this program           *)
(*    with the OCaml compiler and distribute executables, as long as you         *)
(*    follow the requirements of the GNU GPL in regard to all of the             *)
(*    software in the executable aside from the OCaml compiler.                  *)
(*                                                                               *)
(*    Contact: Maxence.Guesdon@inria.fr                                          *)
(*                                                                               *)
(*********************************************************************************)

(** *)

type t =
  { ignored : string list ; (** list of regexps of filenames to ignore *)
    documents : string list ; (** list of regexps for document files *)
    not_documents : string list ;
    (** list of regexps for file matching documents rules but not being  documents *)
    follow_symlinks : bool ;
    levels : (string * (string * int list) list) list ;
  }
;;

module CF = Config_file;;

let config_dir dir = Filename.concat dir ".stog";;
let config_file dir = Filename.concat (config_dir dir) "config";;
let tmpl_dir dir = Filename.concat (config_dir dir) "templates";;
let cache_dir dir = Filename.concat (config_dir dir) "cache";;
let modules_dir dir = Filename.concat (config_dir dir) "modules";;

let read_config dir =
  let rc_file = config_file dir in
  let group = new CF.group in
  let o_ignored = new CF.list_cp CF.string_wrappers ~group
    ["ignored"] [ ".*\\.git" ; ".*Makefile" ; ".*tmpl$" ] "Regexps of files to ignore"
  in
  let o_docs = new CF.list_cp CF.string_wrappers ~group
    ["documents"] [ ".*\\.xml$" ; ".*\\.html$" ] "Regexps of files containing documents"
  in
  let o_not_docs = new CF.list_cp CF.string_wrappers ~group
    ["not-documents"] [ ]
    "Regexps of files matching 'documents' regexps but not containing documents"
  in
  let follow_symlinks = new CF.bool_cp ~group ["follow-symlinks"] false
    "follow symlinks when bulding source file and directory tree"
  in
  let levels = new CF.list_cp
    (CF.tuple2_wrappers CF.string_wrappers
     (CF.list_wrappers (CF.tuple2_wrappers CF.string_wrappers (CF.list_wrappers CF.int_wrappers))))
     ~group ["levels"] []
     "Run levels of modules, for example:
      [ (\"html\", [ (\"base\", [ 0 ; 61 ]) ] ) ]"
  in
  let cfg_dir = config_dir dir in
  if not (Sys.file_exists cfg_dir) then
    begin
      Stog_msg.warning
        (Printf.sprintf "Creating non-existent configuration directory %S" cfg_dir);
      Stog_misc.safe_mkdir cfg_dir;
    end;
  group#read rc_file;
  { ignored = o_ignored#get ;
    documents = o_docs#get ;
    not_documents = o_not_docs#get ;
    follow_symlinks = follow_symlinks#get ;
    levels = levels#get ;
  }
;;




    
